@@ -1,5 +1,5 @@
-//  BMPx - The Dumb Music Player
-//  Copyright (C) 2005 BMPx development team.
+//  BMP
+//  Copyright (C) 2005 BMP development.
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -24,15 +24,21 @@
 
 #include <glibmm.h>
 
-#include <src/util_string.hh>
-#include <src/util_file.hh>
-#include <src/uri++.hh>
+// BMP Audio
+#include "src/audio/audio.hh"
 
-#include <src/database.hh>
-#include <src/vfs.hh>
+// BMP Types
+#include "bmp/types/types-library.hh"
+
+// BMP Misc
+#include "src/util-string.hh"
+#include "src/util-file.hh"
+#include "src/uri.hh"
+#include "src/vfs.hh"
+#include "src/database.hh"
+
+using namespace Glib;
 
-#include <src/x_library.hh>
-#include <src/x_play.hh>
 
 namespace Bmp
 {
@@ -44,48 +50,39 @@ namespace Bmp
         public:
 
           virtual bool
-          can_process (Glib::ustring const& uri)
+          can_process (ustring const& uri)
           {
-            Bmp::URI u (uri);
-            if (u.get_protocol () != Bmp::URI::PROTOCOL_FILE) return false;
-            try
-              {
+            try{
+                Bmp::URI u (uri);
+                if (u.get_protocol () != Bmp::URI::PROTOCOL_FILE) return false;
                 u.unescape ();
-                Glib::ustring uri ((u));
-                std::string path = Glib::filename_from_uri (uri);
-                if (Glib::file_test (path, Glib::FILE_TEST_IS_DIR))
-                  {
-                    return true;
-                  }
+                ustring uri ((u));
+                std::string path = filename_from_uri (uri);
+                return file_test (path, FILE_TEST_IS_DIR);
               }
-            catch (...) {}
+            catch (...) { return false; }
             return false;
           }
 
-          virtual bool 
-          handle_read	(Handle & handle, VUri & list)
+          virtual void 
+          handle_read	(Handle & handle, VUri & list) throw (ProcessingError)
           {
             Bmp::URI u (handle.get_uri());
             u.unescape ();
-            Glib::ustring uri ((u));
-            try
-              {
+            ustring const& uri ((u));
+            try{
                 Util::FileList file_list;
-                Util::collect_path (Glib::filename_from_uri (uri), file_list);
+                Util::collect_audio_paths (filename_from_uri (uri), file_list);
 
-                for (Util::FileList::const_iterator iter = file_list.begin (), end = file_list.end (); iter != end; ++iter)
+                for (Util::FileList::const_iterator i = file_list.begin (); i != file_list.end (); ++i)
                 { 
-                  if (play->is_audio_file ((*iter)))
+                  if (::Bmp::Audio::is_audio_file ((i.operator*())))
                     {
-                      list.push_back (Glib::filename_to_uri (*iter));
+                      list.push_back (filename_to_uri (i.operator*()));
                     }
                 }
-                return true;
-              }
-            catch (...)
-              {
-                return false;
               }
+            catch (...) {}
           }
 
           virtual bool
@@ -94,13 +91,13 @@ namespace Bmp
             return false;
           }
 
-          virtual bool
-          handle_write  (Handle &	handle, VUri const& uri_list)
+          virtual void
+          handle_write  (Handle &	handle, TrackV const& list)
+            throw (ProcessingError)
           {
-            return false;
           }
 
-          virtual Bmp::VFS::ExportData
+          virtual Bmp::VFS::ExportData const&
           get_export_data ()
           {
             static Bmp::VFS::ExportData export_data ("Folder Container Plugin", ""); 
