@@ -1,5 +1,5 @@
-//  BMPx - The Dumb Music Player
-//  Copyright (C) 2005 BMPx development team.
+//  BMP
+//  Copyright (C) 2005-2007 BMP development.
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License Version 2
@@ -28,7 +28,9 @@
 #include <iostream>
 #include <cstring>
 #include <cstdio>
+#include <string>
 
+#include <gtkmm.h>
 #include <glibmm.h>
 #include <glibmm/i18n.h>
 
@@ -41,18 +43,24 @@
 
 #include <boost/format.hpp>
 #include <boost/algorithm/string.hpp>
-#include "uri++.hh"
 
-#include "database.hh"
-#include "x_service_core.hh"
+#include "core.hh"
 
 #include "debug.hh"
 #include "paths.hh"
-
-#include "hal.hh"
+#include "database.hh"
+#include "uri.hh"
 #include "hal++.hh"
 
+#include "bmp/types/types-library.hh"
+
+#include "hal.hh"
+
+using namespace Glib;
+using namespace Gtk;
 using namespace Hal;
+using namespace std;
+using namespace Bmp::DB;
 
 namespace
 {
@@ -96,98 +104,75 @@ namespace
 
 namespace Bmp
 {
-  namespace Library
-  {
-      ::Bmp::DB::DatumDefine hal_volumes_attributes[] = 
+      AttributeInfo
+      volume_attributes[] = 
       {
-          {
-              N_("Volume UDI"),
-              N_("Volume UDIs"),
-              N_("The HAL UDI of the volume"),
-              "volume_udi",
-              DB::VALUE_TYPE_STRING,
-          },
-
-          {
-              N_("Device UDI"),
-              N_("Device UDIs"),
-              N_("The HAL UDI of the device the volume is on"),
-              "device_udi",
-              DB::VALUE_TYPE_STRING,
-          },
-
-          {
-              N_("Mount Path"),
-              N_("Mount Paths"),
-              N_("The current mount path of the volume"),
-              "mount_point",
-              DB::VALUE_TYPE_STRING,
-          },
-
-          {
-              N_("Device Serial"),
-              N_("Device Serials"),
-              N_("The device's serial number"),
-              "drive_serial",
-              DB::VALUE_TYPE_STRING,
-          },
-
-          {
-              N_("Volume Name"),
-              N_("Volume Names"),
-              N_("The name of the volume"),
-              "label",
-              DB::VALUE_TYPE_STRING,
-          },
-
-          {
-              N_("Device File"),
-              N_("Device Files"),
-              N_("The UNIX Device File of this volume"),
-              "device_node",
-              DB::VALUE_TYPE_STRING,
-          },
-
-          {
-              N_("Drive Bus"),
-              N_("Drive Buses"),
-              N_("The device bus the volume's device is on (e.g. USB)"),
-              "drive_bus",
-              DB::VALUE_TYPE_INT,
-          },
-
-          {
-              N_("Drive Type"),
-              N_("Drive Types"),
-              N_("The type of the drive this volume is on"),
-              "drive_type",
-              DB::VALUE_TYPE_INT,
-          },
-
-          {
-              N_("Volume Size"),
-              N_("Volume Sizes"),
-              N_("The storage size of the volume"), 
-              "size",
-              DB::VALUE_TYPE_INT,
-          },
-
-          {
-              N_("Drive Size"),
-              N_("Drive Sizes"),
-              N_("The storage size of the drive this volume is on"), 
-              "drive_size",
-              DB::VALUE_TYPE_INT,
-          },
-
-          {
-              N_("Drive Size"),
-              N_("Drive Sizes"),
-              N_("The storage size of the drive this volume is on"), 
-              "mount_time",
-              DB::VALUE_TYPE_INT,
-          },
-
+        {
+          N_("Volume UDI"),
+          "volume_udi",
+          DB::VALUE_TYPE_STRING,
+          true	
+        },
+        {
+          N_("Device UDI"),
+          "device_udi",
+          DB::VALUE_TYPE_STRING,
+      	  true	
+        },
+        {
+          N_("Mount Path"),
+          "mount_point",
+          DB::VALUE_TYPE_STRING,
+      	  true	
+        },
+        {
+          N_("Device Serial"),
+          "drive_serial",
+          DB::VALUE_TYPE_STRING,
+      	  true	
+        },
+        {
+          N_("Volume Name"),
+          "label",
+          DB::VALUE_TYPE_STRING,
+      	  true	
+        },
+        {
+          N_("Device File"),
+          "device_node",
+          DB::VALUE_TYPE_STRING,
+      	  true	
+        },
+        {
+          N_("Drive Bus"),
+          "drive_bus",
+          DB::VALUE_TYPE_INT,
+      	  true	
+        },
+        {
+          N_("Drive Type"),
+          "drive_type",
+          DB::VALUE_TYPE_INT,
+      	  true	
+        },
+        {
+          N_("Volume Size"),
+          "size",
+          DB::VALUE_TYPE_INT,
+       	  true	
+        },
+        {
+          N_("Drive Size"),
+          "drive_size",
+          DB::VALUE_TYPE_INT,
+      	  true	
+        },
+        {
+          N_("Drive Size"),
+          "mount_time",
+          DB::VALUE_TYPE_INT,
+      	  true	
+        },
       };
 
       enum VolumeAttribute
@@ -208,9 +193,6 @@ namespace Bmp
       HAL::Volume::Volume ( Hal::RefPtr<Hal::Context> const& context,
                             Hal::RefPtr<Hal::Volume>  const& volume) throw ()
       {
-        using namespace Bmp::DB;
-        using namespace Glib; 
-
         Hal::RefPtr<Hal::Drive> drive = Hal::Drive::create_from_udi (context, volume->get_storage_device_udi());
 
         volume_udi      = volume->get_udi();
@@ -232,54 +214,51 @@ namespace Bmp
 #endif //HAVE_HAL_058
 
         drive_serial    = drive->get_serial(); 
-        device_file     = drive->get_device_file(); 
+        device_file     = volume->get_device_file(); 
       }
 
 
       HAL::Volume::Volume (Bmp::DB::Row const& row)
       {
-        using namespace Bmp::DB;
-        using namespace Glib; 
-
         Row::const_iterator i;
         Row::const_iterator end = row.end();
 
-        i = row.find (hal_volumes_attributes[VATTR_VOLUME_UDI].id);
-        volume_udi = boost::get <std::string> (i->second).c_str();
+        i = row.find (volume_attributes[VATTR_VOLUME_UDI].id);
+        volume_udi = boost::get <string> (i->second).c_str();
 
-        i = row.find (hal_volumes_attributes[VATTR_DEVICE_UDI].id);
-        device_udi = boost::get <std::string> (i->second).c_str();
+        i = row.find (volume_attributes[VATTR_DEVICE_UDI].id);
+        device_udi = boost::get <string> (i->second).c_str();
 
-        i = row.find (hal_volumes_attributes[VATTR_DRIVE_BUS].id);
-        drive_bus = Hal::DriveBus(boost::get <uint64_t> (i->second));
+        i = row.find (volume_attributes[VATTR_DRIVE_BUS].id);
+        drive_bus = Hal::DriveBus(boost::get <guint64> (i->second));
 
-        i = row.find (hal_volumes_attributes[VATTR_DRIVE_TYPE].id);
-        drive_type = Hal::DriveType (boost::get <uint64_t> (i->second));
+        i = row.find (volume_attributes[VATTR_DRIVE_TYPE].id);
+        drive_type = Hal::DriveType (boost::get <guint64> (i->second));
 
-        i = row.find (hal_volumes_attributes[VATTR_SIZE].id);
-        size = boost::get <uint64_t> (i->second);
+        i = row.find (volume_attributes[VATTR_SIZE].id);
+        size = boost::get <guint64> (i->second);
 
-        i = row.find (hal_volumes_attributes[VATTR_DRIVE_SIZE].id);
-        drive_size = boost::get <uint64_t> (i->second);
+        i = row.find (volume_attributes[VATTR_DRIVE_SIZE].id);
+        drive_size = boost::get <guint64> (i->second);
 
-        i = row.find (hal_volumes_attributes[VATTR_MOUNT_TIME].id);
-        mount_time = boost::get <uint64_t> (i->second);
+        i = row.find (volume_attributes[VATTR_MOUNT_TIME].id);
+        mount_time = boost::get <guint64> (i->second);
 
-        i = row.find (hal_volumes_attributes[VATTR_MOUNT_PATH].id);
+        i = row.find (volume_attributes[VATTR_MOUNT_PATH].id);
         if (i != end)
-          mount_point = boost::get <std::string> (i->second).c_str();
+          mount_point = boost::get <string> (i->second).c_str();
 
-        i = row.find (hal_volumes_attributes[VATTR_DEVICE_SERIAL].id);
+        i = row.find (volume_attributes[VATTR_DEVICE_SERIAL].id);
         if (i != end)
-          drive_serial = boost::get <std::string> (i->second).c_str();
+          drive_serial = boost::get <string> (i->second).c_str();
 
-        i = row.find (hal_volumes_attributes[VATTR_VOLUME_NAME].id);
+        i = row.find (volume_attributes[VATTR_VOLUME_NAME].id);
         if (i != end)
-          label = boost::get <std::string> (i->second).c_str();
+          label = boost::get <string> (i->second).c_str();
 
-        i = row.find (hal_volumes_attributes[VATTR_DEVICE_FILE].id);
+        i = row.find (volume_attributes[VATTR_DEVICE_FILE].id);
         if (i != end)
-          device_file = boost::get <std::string> (i->second).c_str();
+          device_file = boost::get <string> (i->second).c_str();
       }
 
       ///////////////////////////////////////////////////////
@@ -287,61 +266,63 @@ namespace Bmp
       void
       HAL::register_update_volume (Volume const& volume)
       {
-        Bmp::DB::Row row;
+        static boost::format insert_volume_f
+        ("INSERT INTO hal_volumes (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s) VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%llu', '%llu', '%llu', '%llu', '%llu');");
 
-        row[hal_volumes_attributes[VATTR_VOLUME_UDI].id]    = volume.volume_udi;
-        row[hal_volumes_attributes[VATTR_DEVICE_UDI].id]    = volume.device_udi;
-        row[hal_volumes_attributes[VATTR_MOUNT_PATH].id]    = volume.mount_point;
-        row[hal_volumes_attributes[VATTR_DEVICE_SERIAL].id] = volume.drive_serial;
-        row[hal_volumes_attributes[VATTR_VOLUME_NAME].id]   = volume.label;
-        row[hal_volumes_attributes[VATTR_DEVICE_FILE].id]   = volume.device_file;
-        row[hal_volumes_attributes[VATTR_DRIVE_BUS].id]     = uint64_t(volume.drive_bus);
-        row[hal_volumes_attributes[VATTR_DRIVE_TYPE].id]    = uint64_t(volume.drive_type);
-        row[hal_volumes_attributes[VATTR_SIZE].id]          = uint64_t(volume.size);
-        row[hal_volumes_attributes[VATTR_DRIVE_SIZE].id]    = uint64_t(volume.drive_size);
-        row[hal_volumes_attributes[VATTR_MOUNT_TIME].id]    = uint64_t(volume.mount_time);
-
-        m_hal_db->add ("hal_volumes", row);
+        m_hal_db->exec_sql ((insert_volume_f
+          % volume_attributes[VATTR_VOLUME_UDI].id
+          % volume_attributes[VATTR_DEVICE_UDI].id
+          % volume_attributes[VATTR_MOUNT_PATH].id
+          % volume_attributes[VATTR_DEVICE_SERIAL].id
+          % volume_attributes[VATTR_VOLUME_NAME].id
+          % volume_attributes[VATTR_DEVICE_FILE].id
+          % volume_attributes[VATTR_DRIVE_BUS].id
+          % volume_attributes[VATTR_DRIVE_TYPE].id
+          % volume_attributes[VATTR_SIZE].id
+          % volume_attributes[VATTR_DRIVE_SIZE].id
+          % volume_attributes[VATTR_MOUNT_TIME].id
+          % volume.volume_udi
+          % volume.device_udi
+          % volume.mount_point
+          % volume.drive_serial
+          % volume.label
+          % volume.device_file
+          % guint64(volume.drive_bus)
+          % guint64(volume.drive_type)
+          % guint64(volume.size)
+          % guint64(volume.drive_size)
+          % guint64(volume.mount_time)).str());
       }
 
       void
       HAL::volumes (VVolumes & volumes) const
       {
-        using namespace Bmp::DB;
+        RowV rows;
+        m_hal_db->get ("hal_volumes", rows);
 
-        VRows v;
-        m_hal_db->get ("hal_volumes", v);
-
-        for (VRows::const_iterator r = v.begin(); r != v.end(); ++r)
+        for (RowV::const_iterator i = rows.begin(); i != rows.end(); ++i)
         {
-          Volume volume ((*r));
+          Volume volume (*i);
           volumes.push_back (volume);
         }
       }
 
       HAL::HAL ()
       {
-        using namespace Bmp::DB;
-  
         m_initialized = hal_init ();
         if (!m_initialized)
-          {
-            Gtk::MessageDialog dialog (_("Volume/Device Management could not be initialized; HAL monitoring of"
-                                         " volumes in the library, permanent addition of items to the library"
-                                         " and other parts of BMP will not be functional.\n\nPlease check"
-                                         " your HAL installation and/or contact your distributor."),
-                                          false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK, true);
-            dialog.run();
-            dialog.hide();
-            return;
-          }
+        {
+          MessageDialog dialog (_("HAL could not be accessed from BMP. This is a fatal error. BMP can not continue "
+                                  "to run without HAL accessible."), false, MESSAGE_ERROR, BUTTONS_OK, true);
+          dialog.run();
+          dialog.hide();
+          throw NotInitializedError();
+        }
       }
 
       void
       HAL::cdrom_policy (Hal::RefPtr<Hal::Volume> const& volume)
       {
-        using namespace Hal;
-
         Hal::RefPtr<Hal::Drive> drive = Hal::Drive::create_from_udi (m_context, volume->get_storage_device_udi ());
 
         if (drive->property_exists  ("info.locked") &&
@@ -355,6 +336,7 @@ namespace Bmp
 
         try{
           drive_type = drive->get_type ();
+          debug ("hal", "Drive type is %d", int (drive_type));
           }
         catch (...)
           {
@@ -368,34 +350,35 @@ namespace Bmp
 
             try{
               props = volume->get_disc_properties();
+              debug ("hal", "Disc properties are %d", int (props)); 
               }
             catch (...)
               {
+                debug ("hal", "Couldn't get disc properties for udi '%s'", volume->get_udi().c_str());
                 return;
               }
 
             std::string device_udi (volume->get_storage_device_udi());
 
-            if ((props & Hal::DISC_HAS_AUDIO) && !(props & Hal::DISC_HAS_DATA))
+            if (props & Hal::DISC_HAS_AUDIO)
             {
-              debug ("hal", "CD is CDDA and has no data: %s at %s",
-                volume->get_udi().c_str(), device_udi.c_str());
+              if (!(props & Hal::DISC_HAS_DATA))
+              {
+                debug ("hal", "CD is CDDA and has no data: %s at %s", volume->get_udi().c_str(), device_udi.c_str());
+              }
+              else
+              {
+                debug ("hal", "CD is CDDA/Mixed-Mode: %s at %s", volume->get_udi().c_str(), device_udi.c_str());
+              }
               signal_cdda_inserted_.emit (volume->get_udi(), volume->get_device_file());
             }
             else
-            if ((props & Hal::DISC_HAS_AUDIO) && (props & Hal::DISC_HAS_DATA))
+            if (props & Hal::DISC_HAS_DATA)
             {
-              debug ("hal", "CD is CDDA/Mixed-Mode: %s at %s",
-                volume->get_udi().c_str(), device_udi.c_str());
-              signal_cdda_inserted_.emit (volume->get_udi(), volume->get_device_file());
+              debug ("hal", "CD/DVD is data-only medium");
             }
             else
-            if ((props & Hal::DISC_HAS_DATA))
-            {
-              debug ("hal", "CD is CDROM");
-            }
-            else
-            if ((props & Hal::DISC_IS_BLANK))
+            if (props & Hal::DISC_IS_BLANK)
             {
               debug ("hal", "CD is a blank medium");
             }
@@ -405,28 +388,20 @@ namespace Bmp
       void
       HAL::process_volume (std::string const &udi)
       {
-        using namespace Bmp;
+        debug ("hal", "Processing volume with UDI '%s'", udi.c_str());
 
         Hal::RefPtr<Hal::Volume> volume = Hal::Volume::create_from_udi (m_context, udi);
 
         if (volume->get_device_file().empty())
         {
-          debug ("hal", "cannot get device file: %s", udi.c_str()); 
+          debug ("hal", "Cannot get device file: %s", udi.c_str()); 
           return;
         }
 
-        if (volume->get_fsusage() != Hal::VOLUME_USAGE_MOUNTABLE_FILESYSTEM)
+        if (volume->is_disc())
         {
-          debug ("hal", "no mountable filesystem for %s", udi.c_str());
-          if (volume->is_disc())
-          {
-            cdrom_policy (volume);
-            return;
-          }
-        }
-        else
-        {
-          process_udi (udi);
+          cdrom_policy (volume);
+          return;
         }
       }
 
@@ -442,40 +417,23 @@ namespace Bmp
         return Glib::ustring (HALBus[volume.drive_bus]);
       }
 
-      Glib::RefPtr<Gdk::Pixbuf> 
-      HAL::get_volume_icon (Volume const& volume) const
+      bool
+      HAL::volume_is_mounted  (VolumeKey const& volume_key) const
       {
-        guint index (volume.drive_type);
-        std::string baseicon;
-
-        if (volume.drive_bus == Hal::DRIVE_BUS_USB)
-          {
-            baseicon = drive_type_data[index].icon_usb  ? drive_type_data[index].icon_usb 
-                                                        : drive_type_data[index].icon_default;
+        try{
+            return (m_volumes_mounted.find (volume_key)->second);
           }
-        else
-        if (volume.drive_bus == Hal::DRIVE_BUS_IEEE1394)
+        catch (...)
           {
-            baseicon = drive_type_data[index].icon_1394 ? drive_type_data[index].icon_1394 
-                                                        : drive_type_data[index].icon_default;
+            throw InvalidVolumeSpecifiedError (volume_key.first);
           }
-        else
-          {
-            baseicon = drive_type_data[index].icon_default;
-          }
-        
-        return Gdk::Pixbuf::create_from_file
-          (Glib::build_filename (BMP_IMAGE_DIR_HAL, baseicon));
       }
 
       bool
       HAL::volume_is_mounted  (Volume const& volume) const
       {
         try{
-
-            Hal::RefPtr<Hal::Volume> volume_hal_instance =
-              Hal::Volume::create_from_udi (m_context, volume.volume_udi);
-
+            Hal::RefPtr<Hal::Volume> volume_hal_instance = Hal::Volume::create_from_udi (m_context, volume.volume_udi);
             return volume_hal_instance->is_mounted();
           }
         catch (...)
@@ -487,25 +445,17 @@ namespace Bmp
       HAL::Volume
       HAL::get_volume_for_uri (Glib::ustring const& uri) const
       {
-        using namespace Glib;
-        using namespace std;
-
-        if (!m_initialized)
-          throw NotInitializedError(); 
-
         string filename (filename_from_uri (uri));
-        string::size_type l = 0;
-
+        string::size_type l1 = 0;
         Volumes::const_iterator m = m_volumes.end();
 
         for (Volumes::const_iterator i = m_volumes.begin (); i != m_volumes.end (); ++i)
         {
-          std::string::size_type lv (i->second.first.mount_point.length());
-
-          if ((i->second.first.mount_point == filename.substr (0, lv)) && (lv > l))
+          std::string::size_type l2 (i->second.first.mount_point.length());
+          if ((i->second.first.mount_point == filename.substr (0, l2)) && (l2 > l1))
           {
             m = i;
-            l = lv;
+            l1 = l2;
           } 
 	      }
 
@@ -518,31 +468,22 @@ namespace Bmp
       std::string
       HAL::get_mount_point_for_volume (std::string const& volume_udi, std::string const& device_udi) const
       {
-        if (!m_initialized)
-          throw NotInitializedError(); 
-
         VolumeKey match (volume_udi, device_udi);
         Volumes::const_iterator v_iter = m_volumes.find (match);
 
         if (v_iter != m_volumes.end())
           return v_iter->second.first.mount_point; 
  
-        throw NoMountPathForUdiError
-            ((boost::format ("No mount path for given volume: device: %s, volume: %s") % device_udi % volume_udi).str());
+        throw NoMountPathForVolumeError ((boost::format ("No mount path for given volume: device: %s, volume: %s") % device_udi % volume_udi).str());
       }
 
       void
       HAL::process_udi (std::string const& udi)
       {
-        using boost::algorithm::split;
-        using boost::algorithm::is_any_of;
-        using boost::algorithm::to_lower;
-        using boost::algorithm::find_last;
-        using boost::iterator_range;
-
         debug ("hal", "Processing UDI '%s'", udi.c_str());  
 
-        Hal::RefPtr<Hal::Volume> volume_hal_instance;
+        Hal::RefPtr <Hal::Volume> volume_hal_instance;
+
         try{
           volume_hal_instance = Hal::Volume::create_from_udi (m_context, udi);
           }
@@ -555,28 +496,40 @@ namespace Bmp
         if (!(volume_hal_instance->get_fsusage() == Hal::VOLUME_USAGE_MOUNTABLE_FILESYSTEM))
           return;
 
-        if (!(volume_hal_instance->is_mounted()))
-          return;
+        Volume volume (m_context, volume_hal_instance);
+        VolumeKey volume_key (volume.volume_udi, volume.device_udi);
 
-        try{
+        VolumesMounted::iterator i (m_volumes_mounted.find (volume_key));
+        if (i != m_volumes_mounted.end())
+          m_volumes_mounted.erase (i);
+        m_volumes_mounted.insert (make_pair (volume_key, volume_hal_instance->is_mounted()));
 
-            Volume volume (m_context, volume_hal_instance);
-            VolumeKey key (volume.volume_udi, volume.device_udi);
+        if (volume_hal_instance->is_mounted())
+          signal_volume_added_.emit (volume);
+        else
+          signal_volume_removed_.emit (volume);
 
-            m_volumes.insert (std::make_pair (key, StoredVolume (volume, volume_hal_instance)));
+        try {
+
+            Volumes::iterator i (m_volumes.find (volume_key));
+            if (i != m_volumes.end())
+              return;
+
+            debug ("hal", "Registering 'volume' cap device with udi '%s'", volume.volume_udi.c_str());
+
+            m_volumes.insert (make_pair (volume_key, StoredVolume (volume, volume_hal_instance)));
 
             if (!volume_hal_instance->is_disc())
-              {
-                register_update_volume (volume);  // we don't store information for discs permanently
-              }
+            {
+              register_update_volume (volume);  // we don't store information for discs permanently
+            }
 
-            Hal::RefPtr<Hal::Volume> const& vol (m_volumes.find (key)->second.second);
+            Hal::RefPtr<Hal::Volume> const& vol (m_volumes.find (volume_key)->second.second);
 
             vol->signal_condition().connect
-              (sigc::mem_fun (this, &Bmp::Library::HAL::device_condition));
-
+              (sigc::mem_fun (this, &Bmp::HAL::device_condition));
             vol->signal_property_modified().connect
-              (sigc::mem_fun (this, &Bmp::Library::HAL::device_property));
+              (sigc::mem_fun (this, &Bmp::HAL::device_property));
 
             signal_volume_added_.emit (volume); 
           }
@@ -586,20 +539,17 @@ namespace Bmp
       bool
       HAL::hal_init ()
       {
-        using namespace Bmp::DB;
-        using namespace Glib;
-
         DBusError error;
         dbus_error_init (&error);
 
         DBusConnection * c = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
 
         if (dbus_error_is_set (&error))
-          {
-            debug ("hal", G_STRLOC ": Error conntecting to DBus.");
-            dbus_error_free (&error);
-            return false;
-          }
+        {
+          debug ("hal", G_STRLOC ": Error conntecting to DBus.");
+          dbus_error_free (&error);
+          return false;
+        }
 
         dbus_connection_setup_with_g_main (c, NULL);
         dbus_connection_set_exit_on_disconnect (c, FALSE);
@@ -615,9 +565,9 @@ namespace Bmp
 
         try{
           m_context->signal_device_added().connect
-            (sigc::mem_fun (this, &Bmp::Library::HAL::device_added));
+            (sigc::mem_fun (this, &Bmp::HAL::device_added));
           m_context->signal_device_removed().connect
-            (sigc::mem_fun (this, &Bmp::Library::HAL::device_removed));
+            (sigc::mem_fun (this, &Bmp::HAL::device_removed));
           }
         catch (...)
           {
@@ -627,30 +577,17 @@ namespace Bmp
           }
 
         Hal::StrV list;
-
         try{
           list = m_context->find_device_by_capability ("volume");
           }
         catch (...)
           {
-            debug ("hal", G_STRLOC ": Hal::Context::get_all_devices failed()");
-            m_context.clear();
+            debug ("hal", G_STRLOC ": Hal::Context::find_device_by_capability() failed");
             return false;
           }
 
-        for (unsigned int n = 0; n < G_N_ELEMENTS(hal_volumes_attributes); ++n)
-          {
-            m_hal_map.insert (std::make_pair (std::string (hal_volumes_attributes[n].id), hal_volumes_attributes[n].type));
-          }
-
         m_hal_db = new ::Bmp::DB::Database ("hal", BMP_PATH_DATA_DIR, ::Bmp::DB::DB_OPEN);
 
-        if (!m_hal_db->table_exists ("hal_volumes"))
-        {
-          delete m_hal_db;
-          m_hal_db = new ::Bmp::DB::Database ("hal", BMP_PATH_DATA_DIR, ::Bmp::DB::DB_TRUNCATE);
-        }
-  
         try {
 
             static boost::format sql_create_table_f
@@ -658,11 +595,11 @@ namespace Bmp
             static boost::format sql_attr_f ("'%s'");
   
             std::string attrs;
-            for (unsigned int n = 0; n < G_N_ELEMENTS(hal_volumes_attributes); ++n)
+            for (unsigned int n = 0; n < G_N_ELEMENTS(volume_attributes); ++n)
             {
-              attrs.append((sql_attr_f % hal_volumes_attributes[n].id).str());
+              attrs.append((sql_attr_f % volume_attributes[n].id).str());
 
-              switch (hal_volumes_attributes[n].type)
+              switch (volume_attributes[n].type)
               {
                 case VALUE_TYPE_STRING:
                     attrs.append (" TEXT ");
@@ -675,25 +612,60 @@ namespace Bmp
                 default: break;
               }
 
-              if (n < (G_N_ELEMENTS(hal_volumes_attributes)-1))
+              if (n < (G_N_ELEMENTS(volume_attributes)-1))
                 attrs.append (", ");
             }
 
-            m_hal_db->sqlite_exec_simple ((sql_create_table_f
+            m_hal_db->exec_sql ((sql_create_table_f
                                           % "hal_volumes"
                                           % attrs
-                                          % hal_volumes_attributes[VATTR_DEVICE_UDI].id
-                                          % hal_volumes_attributes[VATTR_VOLUME_UDI].id).str());
-          }
-        catch (Bmp::DB::SqlError& cxe)
-          {
-            abort ();
-          }
+                                          % volume_attributes[VATTR_DEVICE_UDI].id
+                                          % volume_attributes[VATTR_VOLUME_UDI].id).str());
+        }
+        catch (Bmp::DB::SqlExceptionC & cxe)
+        {
+          abort ();
+        }
 
-        m_hal_db->insert_map_for_table ("hal_volumes", m_hal_map);
+        RowV rows;
+        m_hal_db->get ("hal_volumes", rows);
+        for (RowV::const_iterator i = rows.begin(); i != rows.end(); ++i)
+        {
+          Volume const& volume = *i;
+          process_udi (volume.volume_udi);
+        }
 
         for (Hal::StrV::const_iterator n = list.begin(); n != list.end(); ++n) 
+        {
           process_udi (*n);
+        }
+
+        // Get all optical devices
+      
+        try{
+          list = m_context->find_device_by_capability ("storage.cdrom");
+          }
+        catch (...)
+          {
+            debug ("hal", G_STRLOC ": Hal::Context::find_device_by_capability() failed");
+            return false;
+          }
+
+        for (Hal::StrV::const_iterator n = list.begin(); n != list.end(); ++n) 
+        {
+            std::string product, dev; 
+            try{
+                Hal::RefPtr<Hal::Device> device = Hal::Device::create(m_context, *n);
+                product = device->get_property<std::string>("info.product");
+                dev = device->get_property<std::string>("block.device");
+                debug ("hal", G_STRLOC ": Optical device '%s' at device '%s'", product.c_str(), dev.c_str());
+               } 
+            catch (...)
+               {
+                debug ("hal", G_STRLOC ": Couldn't create device for UDI '%s'", product.c_str());
+                return false;
+               }
+        }
 
         return true;
       }
@@ -718,22 +690,23 @@ namespace Bmp
       void
       HAL::device_added (std::string const& udi)
       {
+        debug ("hal", "Device added: '%s'", udi.c_str());
+
         try{
-            if (!m_context->device_query_capability (udi, "volume"))
-              return;
-            process_volume (udi);       
+            if (m_context->device_query_capability (udi, "volume"))
+            {
+              process_volume (udi);       
+            }
           }
         catch (...)
           {
             debug ("hal", "Error in device-added signal handler!");
-            return;
           }
       }
 
       void
       HAL::device_removed (std::string const& udi) 
       {
-        //FIXME: Remove from m_volumes 
         signal_device_removed_.emit (udi);
         debug ("hal", "device removed: %s", udi.c_str());
       }
@@ -745,9 +718,6 @@ namespace Bmp
            bool               is_removed,
            bool               is_added)
       {
-        using namespace Bmp;
-        using namespace Bmp::Library;
-
         if (!m_context->device_query_capability (udi, "volume"))
         {
           debug ("hal", "'%s' does not have 'volume' capability!");
@@ -755,22 +725,8 @@ namespace Bmp
         }
 
         Hal::RefPtr<Hal::Volume> volume = Hal::Volume::create_from_udi (m_context, udi);
-
-        debug ("hal", "'%s' is %s", (volume->is_mounted()?"mounted":"not mounted"));
-
-        if (volume->is_mounted())
-          {
-            process_udi (udi);
-          }
-        else
-          {
-            Volumes::iterator n (m_volumes.find (VolumeKey (udi, volume->get_storage_device_udi ())));
-            if (n != m_volumes.end())
-            {
-              Volume vol (n->second.first);
-              signal_volume_removed_.emit (vol); 
-            }
-          }
+        debug ("hal", "'%s' is %s", udi.c_str(), (volume->is_mounted()?"mounted":"not mounted"));
+        process_udi (udi);
       }
 
       HAL::SignalVolume&
@@ -802,5 +758,4 @@ namespace Bmp
       {
         return signal_ejected_;
       }
-  }
 }
