@@ -1,5 +1,5 @@
 /*  BMPx - The Dumb Music Player
- *  Copyright (C) 2005 BMPx development team.
+ *  Copyright (C) 2005-2007 BMPx development team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #ifndef BMP_SANITY_HPP
 #define BMP_SANITY_HPP
 
-#include <glib.h>
+#include <glib/gmacros.h>
 
 G_BEGIN_DECLS
 
