@@ -1,5 +1,5 @@
-//  BMPx - The Dumb Music Player
-//  Copyright (C) 2005 BMPx development team.
+//  BMP
+//  Copyright (C) 2005-2007 BMP development.
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -48,11 +48,13 @@
 #include <boost/algorithm/string.hpp>
 #include <boost/regex.hpp>
 
-#include "util.hh"
+#include "debug.hh"
 #include "md5.h"
-#include "uri++.hh"
+#include "minisoup.hh"
+#include "uri.hh"
+#include "util.hh"
 
-#include "neon++/request.hh"
+using namespace Glib;
 
 #define BOOST_REGEX_MATCH_EXTRA 1
 
@@ -61,13 +63,25 @@ namespace
   const struct {
       const char *exp;
       const char *fmt;
-  } regexices[] = {
+  } lastfm_regexes[] = {
 
-     { "(<\\s*div\\s*[^>]*?\\s*>)([^>]*?)(<\\s*/\\s*div\\s*>)",
-       "(?1)(?2)(?3)"},
+     { "(\\[[^\\]]+\\])",
+       "(?1)"},
 
-     { "(<img\\s+src\\s*=\")([^\">]*?)(\"\\s*?/?>)",
-       "Image '$2'"},
+     { "(\\[\\/[^\\]]+\\])",
+       "(?1)"},
+  };
+
+  const struct {
+      const char *exp;
+      const char *fmt;
+  } html_regexes[] = {
+
+     { "(<div.*/div>)",
+       "(?1)"},
+
+     { "(<img[^>]*>)",
+       "(?1)"},
 
      { "(<a\\s+href\\s*=\")([^\">]*?)(\".*?>)([^>]*?)(<\\s*/\\s*a>)",
        "$2\\: \"$4\""},
@@ -77,6 +91,19 @@ namespace
 
      { "(<br\\s*/\\s*>)",
        "(?1\\n)" },
+
+     { "(<ul>)",
+       "(?1\\n)" },
+
+     { "(</ul>)",
+       "(?1)" },
+
+     { "(<li>)",
+       "(?1\\n•\\t)" },
+
+     { "(</li>)",
+       "(?1\\n)" },
+
   };
 
   // This table of RFC822 timezones is from gmime-utils.c of the gmime API */
@@ -133,67 +160,113 @@ namespace Bmp
   {
 
     bool
-    str_has_prefix_nocase (const gchar *str, const gchar *prefix)
+    str_has_prefix_nocase (std::string const& str,
+                           std::string const& prefix)
     {
-        if ((!str) || (!prefix)) return false;
-        return (strncasecmp (str, prefix, std::strlen (prefix)) == 0);
-    }
-
-    bool
-    str_has_suffix_nocase (const gchar *str, const gchar *_suffix)
-    {
-        gchar *suffix_fq;
-        gboolean rv;
-
-        if ((!str) || (!_suffix)) return false;
-
-        suffix_fq = g_strconcat (".", _suffix, NULL);
-        rv = (strcasecmp (str + std::strlen (str) - std::strlen (suffix_fq), suffix_fq) == 0);
-        g_free (suffix_fq);
-        return rv;
-    }
-
-    bool
-    str_has_suffixes_nocase (const char * str,
-                             char * const * suffixes)
-    {
-        char *const *suffix;
-
-        g_return_val_if_fail (str != 0, false);
-        g_return_val_if_fail (suffixes != 0, false);
-
-        for (suffix = suffixes; *suffix; suffix++)
-          {
-            if (str_has_suffix_nocase (str, *suffix)) return true;
-          }
-
+      if (str.empty () || prefix.empty ())
         return false;
+
+      return (g_ascii_strncasecmp (str.c_str (), prefix.c_str (), prefix.length ()) == 0);
     }
 
     bool
-    str_has_suffixes_nocase_on_crack (std::string const& str,
-                                      StrV const& strv)
+    str_has_suffix_nocase (std::string const& str,
+                           std::string const& suffix)
     {
-        if (str.empty())
-          return false;
-
-        for (StrV::const_iterator i = strv.begin() ; i != strv.end() ; ++i)
-          {
-            if (str_has_suffix_nocase (str.c_str(), i->c_str()))
-              return true;
-          }
+      if (str.empty () || suffix.empty ())
         return false;
+
+      return (g_ascii_strcasecmp (str.c_str () + str.length () - suffix.length (), suffix.c_str ()) == 0);
     }
 
     bool
-    match_keys (Glib::ustring const& _h,
-                Glib::ustring const& _n)
+    str_has_suffixes_nocase (std::string const& str,
+                             char const**       suffixes)
+    {
+      if (str.empty () || !suffixes)
+        return false;
+
+      for (char const** suffix = suffixes; *suffix; ++suffix)
+        {
+          if (str_has_suffix_nocase (str, std::string (*suffix)))
+            return true;
+        }
+
+      return false;
+    }
+
+    bool
+    str_has_suffixes_nocase (std::string const& str,
+                             StrV const&        strv)
+    {
+      if (str.empty () || strv.empty ())
+        return false;
+
+      for (StrV::const_iterator i = strv.begin () ; i != strv.end (); ++i)
+        {
+          if (str_has_suffix_nocase (str, *i))
+            return true;
+        }
+
+      return false;
+    }
+
+    std::string
+    stdstrjoin (StrV const& strings, std::string const& delimiter)
+    {
+      std::string result;
+      StrV::const_iterator e = strings.end(); 
+      --e;
+      for (StrV::const_iterator i = strings.begin(); i != strings.end(); ++i) 
+      {
+        result += *i; 
+      	if (i != e)
+      	{
+	        result += delimiter;
+        }
+      }
+      return result;
+    }
+
+    Glib::ustring
+    utf8_string_normalize (Glib::ustring const& in)
+    {
+      StrV uppercased;
+
+      if (in.empty())
+        return Glib::ustring();
+
+      char ** splitted = g_strsplit_set (in.c_str(), " -", -1);
+      int n = 0;
+
+      while (splitted[n])
+      {
+        if (strlen (splitted[n]))
+        {
+          ustring o  = ustring (splitted[n]).normalize().lowercase();
+          ustring ol = o.substr (1);
+          ustring of = ustring (1, o[0]).uppercase();
+          ustring compose = (of + ol);
+          uppercased.push_back (compose);
+        }
+        ++n;
+      }
+
+      g_strfreev (splitted);
+      ustring norm = stdstrjoin (uppercased, " ");
+      return norm; 
+    }
+
+
+    bool
+    match_keys (ustring const& _h,
+                ustring const& _n)
     {
         using boost::algorithm::split;
         using boost::algorithm::is_any_of;
         using boost::algorithm::find_first;
 
-        StrV m;    
+        StrV m;
 
         std::string n (_n.lowercase());
         std::string h (_h.lowercase());
@@ -201,51 +274,51 @@ namespace Bmp
         split( m, n, is_any_of(" ") );
 
         for (StrV::const_iterator i = m.begin (); i != m.end (); ++i)
-          {
-            if (!find_first (h, (*i)))
-              return false;
-          }
+        {
+	        if (!find_first (h, (*i)))
+	          return false;
+        }
 
         return true;
     }
 
-    char*
-    hexify (char *pass, int len)
+    std::string
+    hex_string (void const* data,
+                std::size_t len)
     {
-      char *hash;
-      char *bp;
-      char hexchars[] = "0123456789abcdef";
-      int i;
+      static const char hexchars[] = "0123456789abcdef";
 
-      hash = static_cast<char *> (g_malloc0 (33));
-      bp = hash;
+      guint8 const* byte_array = static_cast<guint8 const*> (data);
 
-      for(i = 0; i < len; i++)
+      std::string str;
+      str.reserve (len * 2);
+
+      for(unsigned int i = 0; i < len; i++)
         {
-          *(bp++) = hexchars[(pass[i] >> 4) & 0x0f];
-          *(bp++) = hexchars[pass[i] & 0x0f];
+          str.push_back (hexchars[byte_array[i] >> 4]);
+          str.push_back (hexchars[byte_array[i] & 0x0f]);
         }
-      *bp = 0;
-      return hash;
+
+      return str;
     }
 
     std::string
-    md5_hex (char* const data, size_t data_size)
+    md5_hex_string (void const* data,
+                    std::size_t len)
     {
-        md5_state_t  md5state;
-        char         md5pword[16];
+        md5_state_t md5state;
+        md5_byte_t  md5pword[16];
 
         md5_init (&md5state);
-        md5_append (&md5state, (unsigned const char *)data, data_size); 
-        md5_finish (&md5state, (unsigned char *)md5pword);
-        char *md5_hex = Util::hexify (md5pword, sizeof(md5pword));
-        std::string md5_hex_std = md5_hex;
-        free (md5_hex);
-        return md5_hex_std;
+        md5_append (&md5state, static_cast<md5_byte_t const*> (data), static_cast<int> (len));
+        md5_finish (&md5state, md5pword);
+
+        return hex_string (md5pword, sizeof (md5pword));
     }
 
-    std::string 
-    get_timestr (time_t t, int gmt)
+    std::string
+    get_timestr (time_t t,
+                 int    gmt)
     {
       struct tm *tm;
       static char buf[30];
@@ -261,28 +334,46 @@ namespace Bmp
       return std::string(buf);
     }
 
-    // HTML Sanitizer based on regex example code Copyright (c) 1998-2002 John Maddock
-    Glib::ustring 
-    sanitize_html (Glib::ustring const& in)
+    // Last.fm metadata markup sanitizer
+    ustring
+    sanitize_lastfm (ustring const& in)
     {
-
       std::string out = in;
-
       try {
-
         boost::regex e1;
+        for (unsigned int n = 0; n < G_N_ELEMENTS(lastfm_regexes); ++n)
+        {
+          e1.assign (lastfm_regexes[n].exp);
+          out = boost::regex_replace (out, e1, lastfm_regexes[n].fmt, boost::match_default | boost::format_all | boost::match_extra);
+        }
+       }
+      catch (boost::regex_error & cxe)
+      {
+        g_warning ("%s: Error during Last.FM Markup sanitize: %s", G_STRLOC, cxe.what());
+      }
+      return out; 
+    }
 
-        for (unsigned int n = 0; n < G_N_ELEMENTS(regexices); ++n)
-          {
-            e1.assign(regexices[n].exp);
-            out = boost::regex_replace(out, e1, regexices[n].fmt,
-                             boost::match_default | boost::format_all | boost::match_extra);
-          }
+    // HTML Sanitizer based on regex example code Copyright (c) 1998-2002 John Maddock
+    ustring
+    sanitize_html (ustring const& in)
+    {
+      std::string out = in;
+      try {
+        boost::regex e1;
+        for (unsigned int n = 0; n < G_N_ELEMENTS(html_regexes); ++n)
+        {
+          e1.assign(html_regexes[n].exp);
+          out = boost::regex_replace(out, e1, html_regexes[n].fmt, boost::match_default | boost::format_all | boost::match_extra);
+        }
 
        }
-      catch (...) {}
+      catch (boost::regex_error & cxe)
+      { 
+        g_warning ("%s: Error during HTML Markup sanitize: %s", G_STRLOC, cxe.what());
+      }
 
-      return Glib::ustring(out);
+      return out; 
     }
 
     // Converts a RFC822 time string to a time_t value
@@ -297,7 +388,7 @@ namespace Bmp
 
       memset (&tm, 0, sizeof(struct tm));
 
-      // We expect at least something like "03 Dec 12 01:38:34" 
+      // We expect at least something like "03 Dec 12 01:38:34"
       // and don't require a day of week or the timezone.
       //
       // The most specific format we expect:  "Fri, 03 Dec 12 01:38:34 CET"
@@ -308,7 +399,7 @@ namespace Bmp
       // We expect English month names, so we set the locale
       oldlocale = g_strdup (setlocale (LC_TIME, NULL));
       setlocale (LC_TIME, "C");
-      
+
       // Standard format with seconds and 4 digit year
       if (0 != (pos = strptime((const char *)date, " %d %b %Y %T", &tm)))
         success = TRUE;
@@ -321,15 +412,15 @@ namespace Bmp
       // Non-standard format without seconds 2 digit year
       else if (0 != (pos = strptime((const char *)date, " %d %b %y %H:%M", &tm)))
         success = TRUE;
-      
+
       while (pos != 0 && *pos != '\0' && isspace((int)*pos)) pos++; // skip whitespaces before timezone
-      
+
       if (0 != oldlocale)
       {
-        setlocale (LC_TIME, oldlocale);	// Reset Locale 
+        setlocale (LC_TIME, oldlocale);	// Reset Locale
         g_free (oldlocale);
       }
-      
+
       if (TRUE == success)
       {
         if((time_t)(-1) != (t = mktime(&tm)))
@@ -338,7 +429,7 @@ namespace Bmp
           //
           // GMT time, with no daylight savings time
           // correction. (Usually, there is no daylight savings
-          // time since the input is GMT.) 
+          // time since the input is GMT.)
           //
 
           t = t - common_parse_rfc822_tz (pos);
@@ -351,14 +442,64 @@ namespace Bmp
           g_warning ("Internal error! time conversion error! mktime failed!\n");
         }
       }
-      
+
       return 0;
     }
 
+    std::string
+    get_date_string (std::string const& in)
+    {
+      int y = 0, m = 0, d = 0;
 
+      const char * z (in.c_str());
 
+      if (strlen (z) == 4)
+        sscanf (z, "%04d", &y);
+      else
+      if (strlen (z) == 6)
+        sscanf (z, "%04d%02d", &y, &m);
+      else
+      if (strlen (z) == 7)
+        sscanf (z, "%04d-%02d", &y, &m);
+      else
+      if (strlen (z) == 8)
+        sscanf (z, "%04d%02d%02d", &y, &m, &d);
+      else
+      if ((strlen (z) == 10) ||  (strlen (z) == 19))
+        sscanf (z, "%04d-%02d-%02d", &y, &m, &d);
 
-    std::string 
+      char result[512];
+
+      struct tm * _tm = g_new0 (struct tm, 1);
+
+      if (y) _tm->tm_year = (y - 1900);
+      if (m) _tm->tm_mon  = m - 1;
+      if (d) _tm->tm_mday = d;
+
+      if (y && m && d)
+      {
+        g_free (_tm);
+        strftime (result, 511, "%d %B %Y", _tm);
+        return locale_to_utf8 (std::string (result));
+      }
+      else
+      if (y && m)
+      {
+        g_free (_tm);
+        strftime (result, 511, "%B %Y", _tm);
+        return locale_to_utf8 (std::string (result));
+      }
+      else
+      if (y)
+      {
+        g_free (_tm);
+        strftime (result, 511, "%Y", _tm);
+        return locale_to_utf8 (std::string (result));
+      }
+      return std::string();
+    }
+
+    std::string
     get_time ()
     {
       struct tm *tm;
@@ -376,42 +517,49 @@ namespace Bmp
       return std::string(buf);
     }
 
-    Glib::RefPtr<Gdk::Pixbuf> 
-    get_image_from_uri (Glib::ustring const& uri)
+    RefPtr<Gdk::Pixbuf>
+    get_image_from_uri (ustring const& uri)
     {
-      Bmp::URI u (uri);
-      Neon::Request request (u.hostname, u.path);
+      RefPtr<Gdk::Pixbuf> image = RefPtr<Gdk::Pixbuf>(0);
 
-      try {
-        request.dispatch ();
-        }
-      catch (Neon::Request::Exception &cxe)
+      try{
+
+        URI u (uri);
+        if (u.get_protocol() == URI::PROTOCOL_HTTP)
         {
-          g_log (G_LOG_DOMAIN, G_LOG_LEVEL_WARNING, "Error loading image from uri %s: %s", uri.c_str(), cxe.what());
-          return Glib::RefPtr<Gdk::Pixbuf>(0);
+          Soup::RequestSyncRefP request = Soup::RequestSync::create (ustring (u));
+          guint code = request->run (); 
+
+          if (code == 200)
+          try{
+                RefPtr<Gdk::PixbufLoader> loader = Gdk::PixbufLoader::create ();
+                loader->write (reinterpret_cast<const guint8*>(request->get_data_raw()), request->get_data_size());
+                loader->close ();
+                image = loader->get_pixbuf();
+            }
+          catch (Gdk::PixbufError & cxe)
+            {
+                debug("utility","%s: Gdk::PixbufError: %s", G_STRLOC, cxe.what().c_str());
+            }
         }
-
-      std::string format;
-
-      if ((str_has_suffix_nocase (uri.c_str(), "jpeg")) || (str_has_suffix_nocase (uri.c_str(), "jpg")))
-          format = "jpeg";
-      else if (str_has_suffix_nocase (uri.c_str(), "gif"))
-          format = "gif";
-      else if (str_has_suffix_nocase (uri.c_str(), "png"))
-          format = "png";
-
-      Glib::RefPtr<Gdk::Pixbuf> image;
-    
-      try
+        else if (u.get_protocol() == URI::PROTOCOL_FILE)
         {
-          Glib::RefPtr<Gdk::PixbufLoader> loader = Gdk::PixbufLoader::create (format);
-          loader->write (reinterpret_cast<const guint8*>(request.get_data()), request.get_data_size());
-          request.clear ();
-          loader->close ();
-          image = loader->get_pixbuf();
+          image = Gdk::Pixbuf::create_from_file (filename_from_uri (uri));
         }
-      catch (...) {}
-        return image;
+      }
+      catch (Bmp::URI::ParseError & cxe)
+      { 
+          debug("utility","%s: URI Parse Error: %s", G_STRLOC, cxe.what());
+      }
+
+      return image;
+    }
+
+    void
+    free_gvalue_ptr (GValue* value)
+    {
+        g_value_unset (value);
+        g_free (value);
     }
   }
 }
